/*
 * This file is a part of PikoTools
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2021-2022, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef headerfile_picotools_convert_baseparser
#define headerfile_picotools_convert_baseparser

#include <string>
#include <fstream>
#include "textstream/textstream.h"


namespace pt
{

class BaseParser
{
protected:

	BaseParser();

	virtual void clear_input_flags();

	virtual void check_new_line();
	virtual int read_utf8_char();
	virtual int read_ascii_char();
	virtual int read_char_from_wchar_string();
	virtual int read_char_from_utf8_string();
	virtual int read_char_from_ascii_string();
	virtual int read_char_from_wtext_stream();
	virtual int read_char_from_utf8_text_stream();
	virtual int read_char_from_ascii_text_stream();
	virtual int read_char_no_escape();
	virtual int read_char();



	/*
		a number of a line in which there is a syntax_error
	*/
	int line;

	/*
		a number of a column in which there is a syntax_error
	*/
	int column;


	/*
		true if parse() method was called
		false if ParseString() was called
	*/
	bool reading_from_file;


	/*
		pointers to the current character
		if ParseString() is in used
	*/
	const char    * pchar_ascii;
	const wchar_t * pchar_unicode;


	/*
		pointers to WTextStream iterators
		if set then both of them should be set
	 */
	WTextStream::const_iterator * wtext_stream_iterator;
	WTextStream::const_iterator * wtext_stream_iterator_end;


	/*
		pointers to TextStream iterators
		if set then both of them should be set
	 */
	TextStream::const_iterator * text_stream_iterator;
	TextStream::const_iterator * text_stream_iterator_end;


	/*
		last read char
		or -1 if the end
	*/
	int lastc;


	/*
		current file

		may it would be better to make a pointer?
		if we parse only a string then there is no sense to have such an object
	*/
	std::ifstream file;


	/*
		input file is in UTF-8
		default: true
	*/
	bool input_as_utf8;



};

}

#endif
